@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-tab.c,v 1.158.2.3 2005/10/02 19:00:12 chpe Exp $
+ *  $Id$
  */
 
 #include "config.h"
@@ -71,6 +71,10 @@
 #define EPHY_TAB_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_TAB, EphyTabPrivate))
 
 #define MAX_HIDDEN_POPUPS 5
+#define MAX_HIDDEN_POPUPS	5
+#define RELOAD_DELAY		250 /* ms */
+#define RELOAD_DELAY_MAX_TICKS	40  /* RELOAD_DELAY * RELOAD_DELAY_MAX_TICKS = 10 s */
+#define MAX_TITLE_LENGTH	512 /* characters */
 
 struct _EphyTabPrivate
 {
@@ -106,7 +110,8 @@ struct _EphyTabPrivate
 
 	/* File watch */
 	GnomeVFSMonitorHandle *monitor;
-	guint reload_scheduled_id;	
+	guint reload_scheduled_id;
+	guint reload_delay_ticks;	
 };
 
 static void ephy_tab_class_init		(EphyTabClass *klass);
@@ -356,17 +361,6 @@ ephy_tab_grab_focus (GtkWidget *widget)
 	gtk_widget_grab_focus (GTK_WIDGET (ephy_tab_get_embed (tab)));
 }
 
-static EphyWindow *
-ephy_tab_get_window (EphyTab *tab)
-{
-	GtkWidget *toplevel;
-
-	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (tab));
-	g_return_val_if_fail (toplevel != NULL, NULL);
-
-	return EPHY_WINDOW (toplevel);
-}
-
 static void
 ephy_tab_class_init (EphyTabClass *class)
 {
@@ -1265,6 +1259,8 @@ ephy_tab_file_monitor_cancel (EphyTab *tab)
 		g_source_remove (priv->reload_scheduled_id);
 		priv->reload_scheduled_id = 0;
 	}
+
+	priv->reload_delay_ticks = 0;
 }
 
 static gboolean
@@ -1273,7 +1269,24 @@ ephy_file_monitor_reload_cb (EphyTab *tab)
 	EphyTabPrivate *priv = tab->priv;
 	EphyEmbed *embed;
 
-	LOG ("Reloading file '%s'\n", priv->address);
+	if (priv->reload_delay_ticks > 0)
+	{
+		priv->reload_delay_ticks--;
+
+		/* Run again */
+		return TRUE;
+	}
+
+	if (priv->is_loading)
+	{
+		/* Wait a bit to reload if we're still loading! */
+		priv->reload_delay_ticks = RELOAD_DELAY_MAX_TICKS / 2;
+
+		/* Run again */
+		return TRUE;
+	}
+
+	LOG ("Reloading file '%s'", priv->address);
 
 	priv->reload_scheduled_id = 0;
 
@@ -1306,15 +1319,25 @@ ephy_tab_file_monitor_cb (GnomeVFSMonitorHandle *handle,
 			 * Delay the reload a little bit so we don't endlessly
 			 * reload while a file is written.
 			 */
-			if (priv->reload_scheduled_id != 0)
+			if (priv->reload_delay_ticks == 0)
 			{
-				g_source_remove (priv->reload_scheduled_id);
+				priv->reload_delay_ticks = 1;
+			}
+			else
+			{
+				/* Exponential backoff */
+				priv->reload_delay_ticks = MIN (priv->reload_delay_ticks * 2,
+								RELOAD_DELAY_MAX_TICKS);
+			}
+
+			if (priv->reload_scheduled_id == 0)
+			{
+				priv->reload_scheduled_id =
+					g_timeout_add (RELOAD_DELAY,
+						       (GSourceFunc) ephy_file_monitor_reload_cb,
+						       tab);
 			}
 
-			priv->reload_scheduled_id =
-				g_timeout_add (100 /* ms */,
-					       (GSourceFunc) ephy_file_monitor_reload_cb,
-					       tab);
 			break;
 
 		case GNOME_VFS_MONITOR_EVENT_DELETED:
@@ -1807,12 +1830,8 @@ static gboolean
 open_link_in_new_tab (EphyTab *tab,
 		      const char *link_address)
 {
-	EphyWindow *window;
 	gboolean new_tab;
 
-	window = ephy_tab_get_window (tab);
-	g_return_val_if_fail (window != NULL, FALSE);
-
 	new_tab = address_has_web_scheme (link_address);
 
 	if (new_tab)
@@ -1996,6 +2015,7 @@ ephy_tab_init (EphyTab *tab)
 	priv->title = NULL;
 	priv->loading_title = NULL;
 	priv->address_expire = EPHY_TAB_ADDRESS_EXPIRE_NOW;
+	priv->reload_delay_ticks = 0;
 
 	embed = ephy_embed_factory_new_object (EPHY_TYPE_EMBED);
 	g_assert (embed != NULL);
